#!/usr/bin/env python
# encoding: iso-8859-1

from iixr.files import *
from iixr.terms import *
from iixr.index import *
import os, sys

# Remove old test files.

for filename in ("test", "testMS", "testNMS", "testP", "testP2"):
    try:
        os.remove(filename)
    except OSError:
        pass

try:
    for dirname in ("test_index",):
        for filename in os.listdir(dirname):
            os.remove(os.path.join(dirname, filename))
        os.rmdir(dirname)
except OSError:
    pass

if "clean" in sys.argv:
    sys.exit(0)

print "- Test basic data types."

numbers = [12345678, 0, 1, 127, 128, 255, 256]

f = open("test", "wb")
w = FileWriter(f)
w.begin_record()
for number in numbers:
    w.write_number(number)
w.end_record()
w.close()

f = open("test", "rb")
r = FileReader(f)
r.begin_record()
for number in numbers:
    n = r.read_number()
    print number == n, number, n
r.end_record()
r.close()

tuples = [(0, 0), (1, 3), (2, 5), (3, 9)]

f = open("testMS", "wb")
w = FileWriter(f)
w.begin_record()
w.write_monotonic_sequence(tuples, 2)
w.end_record()
w.close()

f = open("testMS", "rb")
r = FileReader(f)
r.begin_record()
for t, t2 in zip(r.read_monotonic_sequence(2), tuples):
    print t == t2, t, t2
r.end_record()
r.close()

tuples2 = [(0, 0), (1, 3), (2, 1), (3, 2), (4, 0)]

f = open("testNMS", "wb")
w = FileWriter(f)
w.begin_record()
w.write_delta_sequence(tuples2, 2)
w.end_record()
w.close()

f = open("testNMS", "rb")
r = FileReader(f)
r.begin_record()
for t, t2 in zip(r.read_delta_sequence(2), tuples2):
    print t == t2, t, t2
r.end_record()
r.close()

print "- Test positions."

all_doc_positions = [
    [
        (123, [1, 3, 5, 15, 25]),
        (124, [0, 100]),
        (125, [11, 99, 199]),
        (130, [77, 78, 80, 82, 89])
    ],
    [
        (78, [9]),
        (196, [10, 11]),
        (197, [17, 21, 30])
    ]
    ]

f = open("testP", "wb")
w = TermWriter(f)
w.begin(0, 0)
for doc_positions in all_doc_positions:
    w.write_positions(doc_positions)
    w.end_record()
w.close()

f = open("testP", "rb")
r = TermReader(f)
for doc_positions in all_doc_positions:
    r.begin_record()
    dp = r.read_positions()
    print doc_positions == dp, doc_positions
    print "     ", dp
r.close()

all_doc_positions_seq = [
    [
        ((123, 0), [(1, 5), (3, 9), (5, 15), (15, 45), (25, 70)]),
        ((124, 1), [(0, 0), (100, 350)]),
        ((124, 2), [(11, 38), (99, 379), (199, 720)]),
        ((130, 0), [(77, 286), (78, 290), (80, 300), (82, 304), (89, 316)])
    ],
    [
        ((78, 1), [(9, 19)]),
        ((196, 0), [(10, 27), (11, 29)]),
        ((196, 1), [(17, 46), (21, 52), (30, 60)])
    ]
    ]

f = open("testP2", "wb")
w = TermWriter(f)
w.begin(2, 2)
for doc_positions in all_doc_positions_seq:
    w.write_positions(doc_positions)
    w.end_record()
w.close()

f = open("testP2", "rb")
r = TermReader(f)
for doc_positions in all_doc_positions_seq:
    r.begin_record()
    dp = r.read_positions()
    print doc_positions == dp, doc_positions
    print "     ", dp
r.close()

print "- Test dictionaries with term and position data."

terms_with_positions = [
    ("aardvark",  [(1, [2, 45, 96]), (20, [13])]),
    ("anteater",  [(1, [43, 44])]),
    ("badger",    [(7, [2, 22, 196]), (19, [55, 1333]), (21, [0])]),
    (u"bjrn",    [(11, [19, 54])]),
    ("bull",      [(6, [128]), (16, [12]), (26, [1, 3, 5, 7, 9]), (36, [2, 4, 6, 8, 10])]),
    ("bulldog",   [(43, [17, 19, 256, 512])]),
    ("cat",       [(123, [12, 145, 196]), (1200, [113])]),
    (u"",        [(15, [384])]),
    ]

f = open("test", "wb")
w = TermWriter(f)
w.begin(0, 0)
w.write_terms(terms_with_positions)
w.close()

f = open("test", "rb")
r = TermIterator(f)
for (term, doc_positions), (t, dp) in zip(terms_with_positions, r):
    print term == t, term, t
    print doc_positions == dp, doc_positions
    print "     ", dp
r.close()

f = open("test", "rb")
r = TermDataIterator(f)
for (term, doc_positions), (t, data) in zip(terms_with_positions, r):
    print term == t, term, t, data
r.close()

print "- Test high-level index operations."

docs = [
    (1, "The cat sat on the mat"),
    (2, "Every good boy deserves football"),
    (13, "One good turn deserves another"),
    (14, "Every man for himself"),
    (25, "Red sky at night shepherd's delight"),
    (36, "She sells sea shells on the sea shore")
    ]

index = Index("test_index", 3)
wi = index.get_writer()
for docnum, text in docs:
    doc = Document(docnum)
    for position, term in enumerate(text.split()):
        doc.add_position(term, position)
    wi.add_document(doc)
wi.close()

print "- Test merge."

l1 = list(index.get_reader())
index.merge(3)
l2 = list(index.get_reader(1))

for (t1, dp1), (t2, dp2) in zip(l1, l2):
    print t1 == t2, t1, t2
    print dp1 == dp1, dp1
    print "     ", dp2

index.close()

# vim: tabstop=4 expandtab shiftwidth=4
